@@ -44,6 +44,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include "glinfo_common.h"
 
 
 #ifndef GLX_NONE_EXT
@@ -62,8 +63,6 @@
 #define GLX_COLOR_INDEX_BIT		0x00000002
 #endif
 
-#define ELEMENTS(array) (sizeof(array) / sizeof(array[0]))
-
 typedef enum
 {
    Normal,
@@ -128,11 +127,18 @@ static const struct { int major, minor; } gl_versions[] = {
    {4, 1},
    {4, 2},
    {4, 3},
+   {4, 4},
    {0, 0} /* end of list */
 };
 
 #define NUM_GL_VERSIONS ELEMENTS(gl_versions)
 
+/**
+ * Version of the context that was created
+ *
+ * 20, 21, 30, 31, 32, etc.
+ */
+static int version;
 
 /**
  * GL Error checking/warning.
@@ -147,157 +153,6 @@ CheckError(int line)
 }
 
 
-/*
- * qsort callback for string comparison.
- */
-static int
-compare_string_ptr(const void *p1, const void *p2)
-{
-   return strcmp(* (char * const *) p1, * (char * const *) p2);
-}
-
-
-/*
- * Print a list of extensions, with word-wrapping.
- */
-static void
-print_extension_list(const char *ext, Bool singleLine)
-{
-   char **extensions;
-   int num_extensions;
-   const char *indentString = "    ";
-   const int indent = 4;
-   const int max = 79;
-   int width, i, j, k;
-
-   if (!ext || !ext[0])
-      return;
-
-   /* count the number of extensions, ignoring successive spaces */
-   num_extensions = 0;
-   j = 1;
-   do {
-      if ((ext[j] == ' ' || ext[j] == 0) && ext[j - 1] != ' ') {
-         ++num_extensions;
-      }
-   } while(ext[j++]);
-
-   /* copy individual extensions to an array */
-   extensions = malloc(num_extensions * sizeof *extensions);
-   if (!extensions) {
-      fprintf(stderr, "Error: malloc() failed\n");
-      exit(1);
-   }
-   i = j = k = 0;
-   while (1) {
-      if (ext[j] == ' ' || ext[j] == 0) {
-         /* found end of an extension name */
-         const int len = j - i;
-
-         if (len) {
-            assert(k < num_extensions);
-
-            extensions[k] = malloc(len + 1);
-            if (!extensions[k]) {
-               fprintf(stderr, "Error: malloc() failed\n");
-               exit(1);
-            }
-
-            memcpy(extensions[k], ext + i, len);
-            extensions[k][len] = 0;
-
-            ++k;
-         };
-
-         i += len + 1;
-
-         if (ext[j] == 0) {
-            break;
-         }
-      }
-      j++;
-   }
-   assert(k == num_extensions);
-
-   /* sort extensions alphabetically */
-   qsort(extensions, num_extensions, sizeof extensions[0], compare_string_ptr);
-
-   /* print the extensions */
-   width = indent;
-   printf("%s", indentString);
-   for (k = 0; k < num_extensions; ++k) {
-      const int len = strlen(extensions[k]);
-      if ((!singleLine) && (width + len > max)) {
-         /* start a new line */
-         printf("\n");
-         width = indent;
-         printf("%s", indentString);
-      }
-      /* print the extension name */
-      printf("%s", extensions[k]);
-
-      /* either we're all done, or we'll continue with next extension */
-      width += len + 1;
-
-      if (singleLine) {
-         printf("\n");
-         width = indent;
-         printf("%s", indentString);
-      }
-      else if (k < (num_extensions -1)) {
-         printf(", ");
-         width += 2;
-      }
-   }
-   printf("\n");
-
-   for (k = 0; k < num_extensions; ++k) {
-      free(extensions[k]);
-   }
-   free(extensions);
-}
-
-
-/**
- * Get list of OpenGL extensions using core profile's glGetStringi().
- */
-static char *
-build_core_profile_extension_list(void)
-{
-   GLint i, n, totalLen;
-   char *buffer;
-   static PFNGLGETSTRINGIPROC glGetStringi_func = NULL;
-
-   if (!glGetStringi_func) {
-      glGetStringi_func = (PFNGLGETSTRINGIPROC)
-         glXGetProcAddressARB((GLubyte *) "glGetStringi");
-   }
-
-   glGetIntegerv(GL_NUM_EXTENSIONS, &n);
-
-   /* compute totalLen */
-   totalLen = 0;
-   for (i = 0; i < n; i++) {
-      const char *ext = (const char *) glGetStringi_func(GL_EXTENSIONS, i);
-      totalLen += strlen(ext) + 1; /* plus a space */
-   }
-
-   buffer = malloc(totalLen + 1);
-   if (buffer) {
-      int pos = 0;
-      for (i = 0; i < n; i++) {
-         const char *ext = (const char *) glGetStringi_func(GL_EXTENSIONS, i);
-         strcpy(buffer + pos, ext);
-         pos += strlen(ext);
-         buffer[pos++] = ' ';
-      }
-      buffer[pos] = '\0';
-   }
-   return buffer;
-}
-
-
-
 static void
 print_display_info(Display *dpy)
 {
@@ -305,330 +160,6 @@ print_display_info(Display *dpy)
 }
 
 
-/**
- * Print interesting limits for vertex/fragment programs.
- */
-static void
-print_program_limits(GLenum target)
-{
-#if defined(GL_ARB_vertex_program) || defined(GL_ARB_fragment_program)
-   struct token_name {
-      GLenum token;
-      const char *name;
-   };
-   static const struct token_name common_limits[] = {
-      { GL_MAX_PROGRAM_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_TEMPORARIES_ARB, "GL_MAX_PROGRAM_TEMPORARIES_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB, "GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB" },
-      { GL_MAX_PROGRAM_PARAMETERS_ARB, "GL_MAX_PROGRAM_PARAMETERS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB, "GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB" },
-      { GL_MAX_PROGRAM_ATTRIBS_ARB, "GL_MAX_PROGRAM_ATTRIBS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB, "GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB" },
-      { GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB, "GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB, "GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB" },
-      { GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB, "GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB" },
-      { GL_MAX_PROGRAM_ENV_PARAMETERS_ARB, "GL_MAX_PROGRAM_ENV_PARAMETERS_ARB" },
-      { (GLenum) 0, NULL }
-   };
-   static const struct token_name fragment_limits[] = {
-      { GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB, "GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB, "GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB" },
-      { GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB, "GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB" },
-      { (GLenum) 0, NULL }
-   };
-
-   PFNGLGETPROGRAMIVARBPROC GetProgramivARB_func = (PFNGLGETPROGRAMIVARBPROC)
-      glXGetProcAddressARB((GLubyte *) "glGetProgramivARB");
-
-   GLint max[1];
-   int i;
-
-   if (target == GL_VERTEX_PROGRAM_ARB) {
-      printf("    GL_VERTEX_PROGRAM_ARB:\n");
-   }
-   else if (target == GL_FRAGMENT_PROGRAM_ARB) {
-      printf("    GL_FRAGMENT_PROGRAM_ARB:\n");
-   }
-   else {
-      return; /* something's wrong */
-   }
-
-   for (i = 0; common_limits[i].token; i++) {
-      GetProgramivARB_func(target, common_limits[i].token, max);
-      if (glGetError() == GL_NO_ERROR) {
-         printf("        %s = %d\n", common_limits[i].name, max[0]);
-      }
-   }
-   if (target == GL_FRAGMENT_PROGRAM_ARB) {
-      for (i = 0; fragment_limits[i].token; i++) {
-         GetProgramivARB_func(target, fragment_limits[i].token, max);
-         if (glGetError() == GL_NO_ERROR) {
-            printf("        %s = %d\n", fragment_limits[i].name, max[0]);
-         }
-      }
-   }
-#endif /* GL_ARB_vertex_program / GL_ARB_fragment_program */
-}
-
-
-/**
- * Print interesting limits for vertex/fragment shaders.
- */
-static void
-print_shader_limits(GLenum target)
-{
-   struct token_name {
-      GLenum token;
-      const char *name;
-   };
-#if defined(GL_ARB_vertex_shader)
-   static const struct token_name vertex_limits[] = {
-      { GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB, "GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB" },
-      { GL_MAX_VARYING_FLOATS_ARB, "GL_MAX_VARYING_FLOATS_ARB" },
-      { GL_MAX_VERTEX_ATTRIBS_ARB, "GL_MAX_VERTEX_ATTRIBS_ARB" },
-      { GL_MAX_TEXTURE_IMAGE_UNITS_ARB, "GL_MAX_TEXTURE_IMAGE_UNITS_ARB" },
-      { GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB, "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB" },
-      { GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB, "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB" },
-      { GL_MAX_TEXTURE_COORDS_ARB, "GL_MAX_TEXTURE_COORDS_ARB" },
-      { (GLenum) 0, NULL }
-   };
-#endif
-#if defined(GL_ARB_fragment_shader)
-   static const struct token_name fragment_limits[] = {
-      { GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB, "GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB" },
-      { GL_MAX_TEXTURE_COORDS_ARB, "GL_MAX_TEXTURE_COORDS_ARB" },
-      { GL_MAX_TEXTURE_IMAGE_UNITS_ARB, "GL_MAX_TEXTURE_IMAGE_UNITS_ARB" },
-      { (GLenum) 0, NULL }
-   };
-#endif
-   GLint max[1];
-   int i;
-
-#if defined(GL_ARB_vertex_shader)
-   if (target == GL_VERTEX_SHADER_ARB) {
-      printf("    GL_VERTEX_SHADER_ARB:\n");
-      for (i = 0; vertex_limits[i].token; i++) {
-         glGetIntegerv(vertex_limits[i].token, max);
-         if (glGetError() == GL_NO_ERROR) {
-            printf("        %s = %d\n", vertex_limits[i].name, max[0]);
-         }
-      }
-   }
-#endif
-#if defined(GL_ARB_fragment_shader)
-   if (target == GL_FRAGMENT_SHADER_ARB) {
-      printf("    GL_FRAGMENT_SHADER_ARB:\n");
-      for (i = 0; fragment_limits[i].token; i++) {
-         glGetIntegerv(fragment_limits[i].token, max);
-         if (glGetError() == GL_NO_ERROR) {
-            printf("        %s = %d\n", fragment_limits[i].name, max[0]);
-         }
-      }
-   }
-#endif
-}
-
-
-/** Is extension 'ext' supported? */
-static int
-extension_supported(const char *ext, const char *extensionsList)
-{
-   const char *p = strstr(extensionsList, ext);
-   if (p) {
-      /* check that next char is a space or end of string */
-      int extLen = strlen(ext);
-      if (p[extLen] == 0 || p[extLen] == ' ')
-         return 1;
-   }
-   return 0;
-}
-
-
-/**
- * Print interesting OpenGL implementation limits.
- */
-static void
-print_limits(const char *extensions, const char *oglstring)
-{
-   struct token_name {
-      GLuint count;
-      GLenum token;
-      const char *name;
-      const char *extension;
-   };
-   static const struct token_name limits[] = {
-      { 1, GL_MAX_ATTRIB_STACK_DEPTH, "GL_MAX_ATTRIB_STACK_DEPTH", NULL },
-      { 1, GL_MAX_CLIENT_ATTRIB_STACK_DEPTH, "GL_MAX_CLIENT_ATTRIB_STACK_DEPTH", NULL },
-      { 1, GL_MAX_CLIP_PLANES, "GL_MAX_CLIP_PLANES", NULL },
-      { 1, GL_MAX_COLOR_MATRIX_STACK_DEPTH, "GL_MAX_COLOR_MATRIX_STACK_DEPTH", "GL_ARB_imaging" },
-      { 1, GL_MAX_ELEMENTS_VERTICES, "GL_MAX_ELEMENTS_VERTICES", NULL },
-      { 1, GL_MAX_ELEMENTS_INDICES, "GL_MAX_ELEMENTS_INDICES", NULL },
-      { 1, GL_MAX_EVAL_ORDER, "GL_MAX_EVAL_ORDER", NULL },
-      { 1, GL_MAX_LIGHTS, "GL_MAX_LIGHTS", NULL },
-      { 1, GL_MAX_LIST_NESTING, "GL_MAX_LIST_NESTING", NULL },
-      { 1, GL_MAX_MODELVIEW_STACK_DEPTH, "GL_MAX_MODELVIEW_STACK_DEPTH", NULL },
-      { 1, GL_MAX_NAME_STACK_DEPTH, "GL_MAX_NAME_STACK_DEPTH", NULL },
-      { 1, GL_MAX_PIXEL_MAP_TABLE, "GL_MAX_PIXEL_MAP_TABLE", NULL },
-      { 1, GL_MAX_PROJECTION_STACK_DEPTH, "GL_MAX_PROJECTION_STACK_DEPTH", NULL },
-      { 1, GL_MAX_TEXTURE_STACK_DEPTH, "GL_MAX_TEXTURE_STACK_DEPTH", NULL },
-      { 1, GL_MAX_TEXTURE_SIZE, "GL_MAX_TEXTURE_SIZE", NULL },
-      { 1, GL_MAX_3D_TEXTURE_SIZE, "GL_MAX_3D_TEXTURE_SIZE", NULL },
-      { 2, GL_MAX_VIEWPORT_DIMS, "GL_MAX_VIEWPORT_DIMS", NULL },
-      { 2, GL_ALIASED_LINE_WIDTH_RANGE, "GL_ALIASED_LINE_WIDTH_RANGE", NULL },
-      { 2, GL_SMOOTH_LINE_WIDTH_RANGE, "GL_SMOOTH_LINE_WIDTH_RANGE", NULL },
-      { 2, GL_ALIASED_POINT_SIZE_RANGE, "GL_ALIASED_POINT_SIZE_RANGE", NULL },
-      { 2, GL_SMOOTH_POINT_SIZE_RANGE, "GL_SMOOTH_POINT_SIZE_RANGE", NULL },
-#if defined(GL_ARB_texture_cube_map)
-      { 1, GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB, "GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB", "GL_ARB_texture_cube_map" },
-#endif
-#if defined(GL_NV_texture_rectangle)
-      { 1, GL_MAX_RECTANGLE_TEXTURE_SIZE_NV, "GL_MAX_RECTANGLE_TEXTURE_SIZE_NV", "GL_NV_texture_rectangle" },
-#endif
-#if defined(GL_ARB_texture_compression)
-      { 1, GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB, "GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB", "GL_ARB_texture_compression" },
-#endif
-#if defined(GL_ARB_multitexture)
-      { 1, GL_MAX_TEXTURE_UNITS_ARB, "GL_MAX_TEXTURE_UNITS_ARB", "GL_ARB_multitexture" },
-#endif
-#if defined(GL_EXT_texture_lod_bias)
-      { 1, GL_MAX_TEXTURE_LOD_BIAS_EXT, "GL_MAX_TEXTURE_LOD_BIAS_EXT", "GL_EXT_texture_lod_bias" },
-#endif
-#if defined(GL_EXT_texture_filter_anisotropic)
-      { 1, GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, "GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT", "GL_EXT_texture_filter_anisotropic" },
-#endif
-#if defined(GL_ARB_draw_buffers)
-      { 1, GL_MAX_DRAW_BUFFERS_ARB, "GL_MAX_DRAW_BUFFERS_ARB", "GL_ARB_draw_buffers" },
-#endif
-#if defined(GL_ARB_blend_func_extended)
-      { 1, GL_MAX_DUAL_SOURCE_DRAW_BUFFERS, "GL_MAX_DUAL_SOURCE_DRAW_BUFFERS", "GL_ARB_blend_func_extended" },
-#endif
-#if defined (GL_ARB_framebuffer_object)
-      { 1, GL_MAX_RENDERBUFFER_SIZE, "GL_MAX_RENDERBUFFER_SIZE", "GL_ARB_framebuffer_object" },
-      { 1, GL_MAX_COLOR_ATTACHMENTS, "GL_MAX_COLOR_ATTACHMENTS", "GL_ARB_framebuffer_object" },
-      { 1, GL_MAX_SAMPLES, "GL_MAX_SAMPLES", "GL_ARB_framebuffer_object" },
-#endif
-      { 0, (GLenum) 0, NULL, NULL }
-   };
-   GLint i, max[2];
-
-   printf("%s limits:\n", oglstring);
-   for (i = 0; limits[i].count; i++) {
-      if (!limits[i].extension ||
-          extension_supported(limits[i].extension, extensions)) {
-         glGetIntegerv(limits[i].token, max);
-         if (glGetError() == GL_NO_ERROR) {
-            if (limits[i].count == 1)
-               printf("    %s = %d\n", limits[i].name, max[0]);
-            else /* XXX fix if we ever query something with more than 2 values */
-               printf("    %s = %d, %d\n", limits[i].name, max[0], max[1]);
-         }
-      }
-   }
-
-   /* these don't fit into the above mechanism, unfortunately */
-   if (extension_supported("GL_ARB_imaging", extensions)) {
-      glGetConvolutionParameteriv(GL_CONVOLUTION_2D, GL_MAX_CONVOLUTION_WIDTH, max);
-      glGetConvolutionParameteriv(GL_CONVOLUTION_2D, GL_MAX_CONVOLUTION_HEIGHT, max+1);
-      printf("    GL_MAX_CONVOLUTION_WIDTH/HEIGHT = %d, %d\n", max[0], max[1]);
-   }
-
-#if defined(GL_ARB_vertex_program)
-   if (extension_supported("GL_ARB_vertex_program", extensions)) {
-      print_program_limits(GL_VERTEX_PROGRAM_ARB);
-   }
-#endif
-#if defined(GL_ARB_fragment_program)
-   if (extension_supported("GL_ARB_fragment_program", extensions)) {
-      print_program_limits(GL_FRAGMENT_PROGRAM_ARB);
-   }
-#endif
-#if defined(GL_ARB_vertex_shader)
-   if (extension_supported("GL_ARB_vertex_shader", extensions)) {
-      print_shader_limits(GL_VERTEX_SHADER_ARB);
-   }
-#endif
-#if defined(GL_ARB_fragment_shader)
-   if (extension_supported("GL_ARB_fragment_shader", extensions)) {
-      print_shader_limits(GL_FRAGMENT_SHADER_ARB);
-   }
-#endif
-}
-
-
-struct bit_info
-{
-   int bit;
-   const char *name;
-};
-
-
-/**
- * Return string representation for bits in a bitmask.
- */
-static const char *
-bitmask_to_string(const struct bit_info bits[], int numBits, int mask)
-{
-   static char buffer[256], *p;
-   int i;
-
-   strcpy(buffer, "(none)");
-   p = buffer;
-   for (i = 0; i < numBits; i++) {
-      if (mask & bits[i].bit) {
-         if (p > buffer)
-            *p++ = ',';
-         strcpy(p, bits[i].name);
-         p += strlen(bits[i].name);
-      }
-   }
-
-   return buffer;
-}
-
-/**
- * Return string representation for the bitmask returned by
- * GL_CONTEXT_PROFILE_MASK (OpenGL 3.2 or later).
- */
-static const char *
-profile_mask_string(int mask)
-{
-   const static struct bit_info bits[] = {
-#ifdef GL_CONTEXT_CORE_PROFILE_BIT
-      { GL_CONTEXT_CORE_PROFILE_BIT, "core profile"},
-#endif
-#ifdef GL_CONTEXT_COMPATIBILITY_PROFILE_BIT
-      { GL_CONTEXT_COMPATIBILITY_PROFILE_BIT, "compatibility profile" }
-#endif
-   };
-
-   return bitmask_to_string(bits, ELEMENTS(bits), mask);
-}
-
-
-/**
- * Return string representation for the bitmask returned by
- * GL_CONTEXT_FLAGS (OpenGL 3.0 or later).
- */
-static const char *
-context_flags_string(int mask)
-{
-   const static struct bit_info bits[] = {
-#ifdef GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT
-      { GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT, "forward-compatible" },
-#endif
-#ifdef GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB
-      { GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB, "robust-access" },
-#endif
-   };
-
-   return bitmask_to_string(bits, ELEMENTS(bits), mask);
-}
-
-
 /**
  * Choose a simple FB Config.
  */
@@ -740,7 +271,7 @@ create_context_flags(Display *dpy, GLXFBConfig fbconfig, int major, int minor,
    if (CreateContextErrorFlag)
       context = 0;
 
-   if (direct) {
+   if (context && direct) {
       if (!glXIsDirect(dpy, context)) {
          glXDestroyContext(dpy, context);
          return 0;
@@ -759,7 +290,7 @@ create_context_flags(Display *dpy, GLXFBConfig fbconfig, int major, int minor,
  */
 static GLXContext
 create_context_with_config(Display *dpy, GLXFBConfig config,
-                           Bool coreProfile, Bool direct)
+                           Bool coreProfile, Bool es2Profile, Bool direct)
 {
    GLXContext ctx = 0;
 
@@ -786,6 +317,19 @@ create_context_with_config(Display *dpy, GLXFBConfig config,
       return 0;
    }
 
+   if (es2Profile) {
+#ifdef GLX_CONTEXT_ES2_PROFILE_BIT_EXT
+      if (extension_supported("GLX_EXT_create_context_es2_profile",
+                              glXQueryExtensionsString(dpy, 0))) {
+         ctx = create_context_flags(dpy, config, 2, 0, 0x0,
+                                    GLX_CONTEXT_ES2_PROFILE_BIT_EXT,
+                                    direct);
+         return ctx;
+      }
+#endif
+      return 0;
+   }
+
    /* GLX should return a context of the latest GL version that supports
     * the full profile.
     */
@@ -831,8 +375,8 @@ choose_xvisinfo(Display *dpy, int scrnum)
 
 static Bool
 print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
-                  Bool coreProfile, Bool limits, Bool singleLine,
-                  Bool coreWorked)
+                  Bool coreProfile, Bool es2Profile, Bool limits,
+                  Bool singleLine, Bool coreWorked)
 {
    Window win;
    XSetWindowAttributes attr;
@@ -842,7 +386,8 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
    XVisualInfo *visinfo;
    int width = 100, height = 100;
    GLXFBConfig *fbconfigs;
-   const char *oglstring = coreProfile ? "OpenGL core profile" : "OpenGL";
+   const char *oglstring = coreProfile ? "OpenGL core profile" :
+                           es2Profile ? "OpenGL ES profile" : "OpenGL";
 
    root = RootWindow(dpy, scrnum);
 
@@ -852,29 +397,30 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
    fbconfigs = choose_fb_config(dpy, scrnum);
    if (fbconfigs) {
       ctx = create_context_with_config(dpy, fbconfigs[0],
-                                       coreProfile, allowDirect);
+                                       coreProfile, es2Profile, allowDirect);
       if (!ctx && allowDirect && !coreProfile) {
          /* try indirect */
          ctx = create_context_with_config(dpy, fbconfigs[0],
-                                          coreProfile, False);
+                                          coreProfile, es2Profile, False);
       }
 
       visinfo = glXGetVisualFromFBConfig(dpy, fbconfigs[0]);
       XFree(fbconfigs);
    }
-   else {
+   else if (!coreProfile && !es2Profile) {
       visinfo = choose_xvisinfo(dpy, scrnum);
       if (visinfo)
 	 ctx = glXCreateContext(dpy, visinfo, NULL, allowDirect);
-   }
+   } else
+      visinfo = NULL;
 
-   if (!visinfo) {
+   if (!visinfo && !coreProfile && !es2Profile) {
       fprintf(stderr, "Error: couldn't find RGB GLX visual or fbconfig\n");
       return False;
    }
 
    if (!ctx) {
-      if (!coreProfile)
+      if (!coreProfile && !es2Profile)
 	 fprintf(stderr, "Error: glXCreateContext failed\n");
       XFree(visinfo);
       return False;
@@ -908,12 +454,21 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
       int glxVersionMinor;
       char *displayName = NULL;
       char *colon = NULL, *period = NULL;
-      int version; /* 20, 21, 30, 31, 32, etc */
+      struct ext_functions extfuncs;
 
       CheckError(__LINE__);
+
+      /* Get some ext functions */
+      extfuncs.GetProgramivARB = (GETPROGRAMIVARBPROC)
+         glXGetProcAddressARB((GLubyte *) "glGetProgramivARB");
+      extfuncs.GetStringi = (GETSTRINGIPROC)
+         glXGetProcAddressARB((GLubyte *) "glGetStringi");
+      extfuncs.GetConvolutionParameteriv = (GETCONVOLUTIONPARAMETERIVPROC)
+         glXGetProcAddressARB((GLubyte *) "glGetConvolutionParameteriv");
+
       /* Get list of GL extensions */
       if (coreProfile) {
-         glExtensions = build_core_profile_extension_list();
+         glExtensions = build_core_profile_extension_list(&extfuncs);
       }
       else {
          glExtensions = (char *) glGetString(GL_EXTENSIONS);
@@ -988,7 +543,7 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
 #endif
       CheckError(__LINE__);
 #ifdef GL_VERSION_3_0
-      if (version >= 30) {
+      if (version >= 30 && !es2Profile) {
          GLint flags;
          glGetIntegerv(GL_CONTEXT_FLAGS, &flags);
          printf("%s context flags: %s\n", oglstring, context_flags_string(flags));
@@ -996,7 +551,7 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
 #endif
       CheckError(__LINE__);
 #ifdef GL_VERSION_3_2
-      if (version >= 32) {
+      if (version >= 32 && !es2Profile) {
          GLint mask;
          glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &mask);
          printf("%s profile mask: %s\n", oglstring, profile_mask_string(mask));
@@ -1010,8 +565,9 @@ print_screen_info(Display *dpy, int scrnum, Bool allowDirect,
 
       CheckError(__LINE__);
 
-      if (limits)
-         print_limits(glExtensions, oglstring);
+      if (limits) {
+         print_limits(glExtensions, oglstring, version, &extfuncs);
+      }
 
       if (coreProfile)
          free(glExtensions);
@@ -1728,8 +1284,9 @@ main(int argc, char *argv[])
       print_display_info(dpy);
       for (scrnum = 0; scrnum < numScreens; scrnum++) {
          mesa_hack(dpy, scrnum);
-         coreWorked = print_screen_info(dpy, scrnum, allowDirect, True, limits, singleLine, False);
-         print_screen_info(dpy, scrnum, allowDirect, False, limits, singleLine, coreWorked);
+         coreWorked = print_screen_info(dpy, scrnum, allowDirect, True, False, limits, singleLine, False);
+         print_screen_info(dpy, scrnum, allowDirect, False, False, limits, singleLine, coreWorked);
+         print_screen_info(dpy, scrnum, allowDirect, False, True, False, singleLine, True);
 
          printf("\n");
          print_visual_info(dpy, scrnum, mode);
